import { Component, EventEmitter, Inject, OnInit, Output } from '@angular/core';
import { FormBuilder, FormGroup, Validators } from '@angular/forms';
import { MAT_DIALOG_DATA, MatDialogRef } from '@angular/material/dialog';
import {ToastrService} from "ngx-toastr";
import {HttpClient} from "@angular/common/http";
import {DataService} from "../data.service";

interface UserGroup {
  '@id': string;
  name: string;
  role: string[];
}

@Component({
  selector: 'app-add-user-modal',
  templateUrl: './add-user-modal.component.html',
  styleUrls: ['./add-user-modal.component.css']
})
export class AddUserModalComponent implements OnInit {
  baseUrl: string = import.meta.env.NG_APP_BASE_API_URL;
  @Output() userAdded = new EventEmitter<void>();
  userForm: FormGroup<any>;
  userGroups: UserGroup[] = [];
  organizationalUnits: any[] = [];
  userId: string | null = null;

  protected views = [
    {value: 'card', name: 'Tarjetas'},
    {value: 'list', name: 'Listado'},
  ];

  constructor(
    public dialogRef: MatDialogRef<AddUserModalComponent>,
    @Inject(MAT_DIALOG_DATA) public data: any,
    private http: HttpClient,
    private fb: FormBuilder,
    private dataService: DataService,
    private toastService: ToastrService
  ) {
    this.userForm = this.fb.group({
      username: ['', Validators.required],
      password: ['', Validators.required],
      role: ['', Validators.required],
      groupsView: ['card', Validators.required],
      organizationalUnits: [[]]
    });
  }

  ngOnInit(): void {
    this.dataService.getUserGroups().subscribe((data) => {
      this.userGroups = data['hydra:member'];
    });
    this.dataService.getOrganizationalUnits().subscribe((data) => {
      this.organizationalUnits = data['hydra:member'].filter((item: any) => item.type === 'organizational-unit');
    });
    if (this.data) {
      this.load()
    }
  }

  load(): void {
    this.dataService.getUser(this.data).subscribe({
      next: (response) => {

        const organizationalUnitIds = response.allowedOrganizationalUnits.map((unit: any) => unit['@id']);

        this.userForm.patchValue({
          username: response.username,
          role: response.userGroups.length > 0 ? response.userGroups[0]['@id'] : null,
          organizationalUnits: organizationalUnitIds,
          groupsView: response.groupsView
        });

        this.userId = response['@id'];
      },
      error: (err) => {
        console.error('Error fetching remote calendar:', err);
      }
    });
  }

  onNoClick(): void {
    this.dialogRef.close();
  }

  onSubmit(): void {
    if (this.userForm.valid) {
      const payload = {
        username: this.userForm.value.username,
        allowedOrganizationalUnits: this.userForm.value.organizationalUnit,
        password: this.userForm.value.password,
        enabled: true,
        userGroups: [this.userForm.value.role ],
        groupsView: this.userForm.value.groupsView
      };

      if (this.userId) {
        this.http.put(`${this.baseUrl}${this.userId}`, payload).subscribe(
          (response) => {
            this.toastService.success('Usuario editado correctamente');
            this.dialogRef.close();
          },
          (error) => {
            this.toastService.error(error['error']['hydra:description']);
            console.error('Error al editar el rol', error);
          }
        );
      } else {
        this.http.post(`${this.baseUrl}/users`, payload).subscribe(
          (response) => {
            this.toastService.success('Usuario añadido correctamente');
            this.dialogRef.close();
          },
          (error) => {
            this.toastService.error(error['error']['hydra:description']);
            console.error('Error al añadir añadido', error);
          }
        );
      }
    }
  }
}
